-- Jarcast Amalfi Functions
-- Author: Jarcast
--==========================================================================================================================
-- INCLUDES
--==========================================================================================================================
WARN_NOT_SHARED = false; include( "SaveUtils" ); MY_MOD_NAME = "Jarcast_TheAmazons";

include("FLuaVector.lua")
include("IconSupport.lua")
include("InstanceManager.lua")
include("AdditionalAchievementsUtility.lua");
--==========================================================================================================================
-- GLOBALS
--==========================================================================================================================
----------------------------------------------------------------------------------------------------------------------------
local Teams 					= Teams

local activePlayerID			= Game.GetActivePlayer()
local activePlayer				= Players[activePlayerID]
local activeTeamID				= activePlayer:GetTeam()
local activeTeam				= Teams[activeTeamID]
--==========================================================================================================================
-- UTILITIES
--==========================================================================================================================
-- UTILITIES
----------------------------------------------------------------------------------------------------------------------------
--Game_IsUsingMod
local CBPmodID = "d1b6328c-ff44-4b0d-aad7-c657f83610cd"
local CBOmodID = "8411a7a8-dad3-4622-a18e-fcc18324c799"
local CSDmodID = "eead0050-1e3f-4178-a91f-26cf1881ac39"
local EEraModID = "ce8aa614-7ef7-4a45-a179-5329869e8d6d"
local EECBPmodID  = "87b2deae-83cb-9b88-9017-79e741d3739c"

function Game_IsUsingMod(modID)
	for _, mod in pairs(Modding.GetActivatedMods()) do
		if mod.ID == modID then
			return true
		end
	end
	return false
end

local isUsingEEra = Game_IsUsingMod(EEraModID) or Game_IsUsingMod(EECBPmodID)
local isUsingCBP  = Game_IsUsingMod(CBPmodID)
local isUsingCBO  = Game_IsUsingMod(CBOmodID)
----------------------------------------------------------------------------------------------------------------------------
--HasTrait
function HasTrait(player, traitID)
	if Game_IsUsingMod(CBPmodID) then 
		return player:HasTrait(traitID)
	else
		local leaderType = GameInfo.Leaders[player:GetLeaderType()].Type
		local traitType  = GameInfo.Traits[traitID].Type
		for row in GameInfo.Leader_Traits("LeaderType = '" .. leaderType .. "' AND TraitType = '" .. traitType .. "'") do
			return true
		end
	end
	return false
end
-------------------------------------------------------------------------------------------------------------------------
-- GetRandom
function GetRandom(lower, upper)
    return Game.Rand((upper + 1) - lower, "") + lower
end
-------------------------------------------------------------------------------------------------------------------------
--IsTraitActive
local slotComputerID = SlotStatus["SS_COMPUTER"]
local slotTakenID = SlotStatus["SS_TAKEN"]
function IsTraitActive(traitID)
	for slotID = 0, GameDefines.MAX_MAJOR_CIVS-1, 1 do
		local slotPlayer = Players[slotID]
		local slotStatus = PreGame.GetSlotStatus(slotID)
		if (slotStatus == slotTakenID or slotStatus == slotComputerID) then
			if HasTrait(slotPlayer, traitID) then
				return true
			end
		end
	end
	return false
end
-------------------------------------------------------------------------------------------------------------------------
if Game then
	--Game.GetUserSetting
	function Game.GetUserSetting(type)
		for row in GameInfo.JFD_GlobalUserSettings("Type = '" .. type .. "'") do
			return row.Value
		end
	end
end
-------------------------------------------------------------------------------------------------------------------------
function C15_IsPlayerAtWarWithAnyone(playerID, bIncludeMinors)
    local pPlayer = Players[playerID]
    local iTeam = pPlayer:GetTeam()
    local pTeam = Teams[iTeam]
    local bIsAtWar = false
    for i = 0, GameDefines.MAX_CIV_PLAYERS - 1 do -- Iterate through everyone
        if i ~= playerID then -- If this isn't the Player, since you can't be at war with yourself...
            local pOtherPlayer = Players[i]
            if pOtherPlayer:IsAlive() then -- Can't be at war with the dead :p
                if not pOtherPlayer:IsBarbarian() then -- You're always at war with Barbs so ignore them
                    if (bIncludeMinors) or ((not bIncludeMinors) and (not pOtherPlayer:IsMinorCiv())) then -- If you've said to include City-States, then everyone will qualify; if you've said to not include them, then make sure pOtherPlayer isn't a City-State
                        local iOtherTeam = pOtherPlayer:GetTeam()
                        if pTeam:IsAtWar(iOtherTeam) then -- The crucial Method
                            bIsAtWar = true
                            break -- We've found someone we're at war with, so we don't need to check for any longer
                        end
                    end
                end
            end
        end
    end
    return bIsAtWar -- Returns either true or false to wherever you called the function
end
-------------------------------------------------------------------------------------------------------------------------
function Civ1HasOpenBorderFromCiv2(iPlayer1, iPlayer2)
	if Teams[Players[iPlayer2]:GetTeam()]:IsAllowsOpenBordersToTeam( Players[iPlayer1]:GetTeam() ) then
        return true
	end
end
-------------------------------------------------------------------------------------------------------------------------
function IsInteger(n)
  return n==math.floor(n)
end
-------------------------------------------------------------------------------------------------------------------------
-- base converter
local floor,insert = math.floor, table.insert
function basen(n,b)
    n = floor(n)
    if not b or b == 10 then return tostring(n) end
    local digits = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    local t = {}
    local sign = ""
    if n < 0 then
        sign = "-"
    n = -n
    end
    repeat
        local d = (n % b) + 1
        n = floor(n / b)
        insert(t, 1, digits:sub(d,d))
    until n == 0
    return sign .. table.concat(t,"")
end
-------------------------------------------------------------------------------------------------------------------------
function Neirai_GetNearestCity(pPlayer, pPlot)
    local distance = 9999
    local cNearestCity = nil
    for cCity in pPlayer:Cities() do
        local pCityPlot = cCity:Plot()
        local between = Map.PlotDistance(pCityPlot:GetX(), pCityPlot:GetY(), pPlot:GetX(), pPlot:GetY())
        if between < distance then
            distance = between
            cNearestCity = cCity
        end
    end
    return cNearestCity
end
-------------------------------------------------------------------------------------------------------------------------
function GetImprovementIDFromResourceID( resourceID)
	tResImprovements  = {}
	for row in DB.Query("SELECT a.ID ResID, b.ResourceType ResType, c.ID ImprovID FROM Resources a, Improvement_ResourceTypes b, Improvements c WHERE a.Type = b.ResourceType and b.ImprovementType = c.Type and a.ID = '" .. resourceID .. "'") do
		table.insert(tResImprovements, row.ImprovID);
	end
	return tResImprovements[1]
end
-------------------------------------------------------------------------------------------------------------------------
function GetResourceClassIDFromResID(resourceID)
	for row in DB.Query("SELECT a.ID ResID, b.ID ResClassID FROM Resources a, ResourceClasses b WHERE a.ResourceClassType = b.Type and a.ID = '" .. resourceID .. "'") do
		return row.ResClassID
	end
end
-------------------------------------------------------------------------------------------------------------------------
function Jar_PlaceResourceOnPlot(playerID, resourceID, numres, pPlot, improvementID)
	if pPlot:GetResourceType() == -1 then
		pPlot:SetResourceType(resourceID, numres)
		if pPlot:IsCity() then
			pPlot:SetImprovementType(GetImprovementIDFromResourceID( resourceID))
			pPlot:SetImprovementType(-1)
			print("Jar_PlaceResourceOnPlot. Resource placed.")
		else
			if pPlot:GetFeatureType() ~= -1 then pAdjacentPlot:SetFeatureType(-1) end
			pPlot:SetImprovementType(-1)
			pPlot:SetImprovementType(improvementID)
			print("Jar_PlaceResourceOnPlot. Resource placed.")
		end
	else
		local plotresID = pPlot:GetResourceType()
		local plotresNum= pPlot:GetNumResource()
		local pCity
		if pPlot:IsCity() then
			pCity = pPlot:GetPlotCity()
		else
			pCity = pPlot:GetWorkingCity()
			if pCity == nil then pCity = Neirai_GetNearestCity(Players[playerID], pPlot) end
		end
		print("Jar_PlaceResourceOnPlot. City found: ".. pCity:GetName())
		local bcheck = true
		for i = 0, pCity:GetNumCityPlots() - 1, 1 do
			local pAdjacentPlot = pCity:GetCityIndexPlot(i)
			if pAdjacentPlot and pAdjacentPlot:GetOwner() == playerID and pAdjacentPlot:GetResourceType() == -1 and (pAdjacentPlot:GetImprovementType() == -1) and not pAdjacentPlot:IsCity() and not pAdjacentPlot:IsWater() and not pAdjacentPlot:IsMountain() then
				if pAdjacentPlot:GetFeatureType() ~= -1 then pAdjacentPlot:SetFeatureType(-1) end
				pAdjacentPlot:SetResourceType(plotresID, plotresNum)
				--pAdjacentPlot:SetImprovementType( GetImprovementIDFromResourceID( plotresID) )
				-- Notification
				local sTitle = "A resource was moved."
				local sText = "A resource was moved to the highlighted plot to make space for another resource to take its place."
				Players[playerID]:AddNotification(NotificationTypes.NOTIFICATION_CITY_TILE, sText, sTitle, pAdjacentPlot:GetX(), pAdjacentPlot:GetY())
				bcheck = false
				break
			end			
		end
		if bcheck then
			--for pAdjacentPlot in PlotAreaSweepIterator(pCity:Plot(), 3, SECTOR_NORTH, DIRECTION_CLOCKWISE, DIRECTION_OUTWARDS, CENTRE_INCLUDE) do
				--x = pAdjacentPlot:GetX()
				--y = pAdjacentPlot:GetY()
			for i = 0, pCity:GetNumCityPlots() - 1, 1 do
				local pAdjacentPlot = pCity:GetCityIndexPlot(i)	
				if pAdjacentPlot and pAdjacentPlot:GetOwner() == -1 and pAdjacentPlot:GetResourceType() == -1 and pAdjacentPlot:GetFeatureType() == -1 and not pAdjacentPlot:IsWater() and not pAdjacentPlot:IsMountain() then
					pAdjacentPlot:SetOwner(playerID, pCity:GetID(), true, true);
					pAdjacentPlot:SetRevealed(Teams[Players[playerID]:GetTeam()], true)
					pAdjacentPlot:SetResourceType(plotresID, plotresNum)
					--pAdjacentPlot:SetImprovementType( GetImprovementIDFromResourceID( plotresID) )
					-- Notification
					local sTitle = "A resource was moved."
					local sText = "A resource was moved to the highlighted plot to make space for another resource to take its place."
					Players[playerID]:AddNotification(NotificationTypes.NOTIFICATION_CITY_TILE, sText, sTitle, pAdjacentPlot:GetX(), pAdjacentPlot:GetY())
					bcheck = false
					break
				end			
			end
		end
		pPlot:SetImprovementType(-1)
		pPlot:SetResourceType(resourceID, numres)
		pPlot:SetImprovementType(improvementID)
		if pPlot:IsCity() then
			pPlot:SetImprovementType( GetImprovementIDFromResourceID( resourceID) )
			pPlot:SetImprovementType(-1)
		end
		print("Jar_PlaceResourceOnPlot. Resource placed.")
	end
end
--==========================================================================================================================
-- GAME DEFINES
--==========================================================================================================================
----------------------------------------------------------------------------------------------------------------------------
--==========================================================================================================================
-- UNIQUE FUNCTIONS
--==========================================================================================================================
-- GLOBALS
--------------------------------------------------------------------------------------------------------------------------
local iMod = ((GameInfo.GameSpeeds[Game.GetGameSpeedType()].BuildPercent)/100)
local iCiv = GameInfoTypes["CIVILIZATION_JAR_AMAZONS"]
local traitAmazonsID = GameInfoTypes["TRAIT_JAR_AMAZONS"]
local g_IsTraitActive = IsTraitActive(traitAmazonsID)
local resHorseID = GameInfoTypes.RESOURCE_HORSE
local promoUAGG = GameInfoTypes["PROMOTION_JAR_UA_GG_RANGE"]
local DummyHorseXP = GameInfoTypes["BUILDING_DUMMY_HORSEXP"]
local iHippomache = GameInfoTypes["UNIT_JAR_HIPPOMACHE"]
local missionHippomacheTrapHorses = GameInfoTypes["MISSION_JAR_HIPPOMACHE_TRAP_HORSES"]
local missionHippomachePlaceHorses = GameInfoTypes["MISSION_JAR_HIPPOMACHE_PLACE_HORSES"]
local promoHorseTrapper = GameInfoTypes["PROMOTION_JAR_HORSE_TRAPPER"]
local iHempSmokeTent = GameInfoTypes["BUILDING_JAR_HEMPSMOKE_TENT"]
local promoCannabis = GameInfoTypes["PROMOTION_JAR_KANNABIS_BOOST"]
local GGclassID = GameInfoTypes["UNITCLASS_GREAT_GENERAL"]
local pastureID = GameInfoTypes.IMPROVEMENT_PASTURE
local promoHorseTrappeD = GameInfoTypes["PROMOTION_JAR_HORSE_TRAPPED"]


local CUSTOM_MISSION_NO_ACTION		 	= 0
local CUSTOM_MISSION_ACTION			 	= 1
local CUSTOM_MISSION_DONE            	= 2
local CUSTOM_MISSION_ACTION_AND_DONE 	= 3

local maxpower = 3
tHorsePromotions = {}
for power=0,maxpower do
	tHorsePromotions[power] = GameInfoTypes["PROMOTION_JAR_HORSE_TRAPPED_" .. tostring(power)]
end
------------------------------------------------------------------------------------------------------------------------
-- QUICK MOD BALANCING PANEL
------------------------------------------------------------------------------------------------------------------------
-- Here are the changeable values that determine the strength of the effects described in the design.
-- UA
local GGPointFromXHorse = 25/100
local XPFromHorse = 2
local XPFromHorseMax = 15
-- UU
local CannabisPromoTurns = 6	-- standard speed as reference
-- UB
local HPHealedGarrison = 5
--========================================================================================================================
-- FUNCTIONS
--========================================================================================================================
-- UA: Daughters of Ares
--========================================================================================================================
function XPandGeneralPointsFromHorses(iPlayer)
	local pPlayer = Players[iPlayer]
	if (HasTrait(pPlayer, traitAmazonsID) and pPlayer:IsAlive()) then
		for pCity in pPlayer:Cities() do
			local cityplot = pCity:Plot()
			local totalhorses = 0
			local numhorsetiles = 0
			for i = 0, pCity:GetNumCityPlots() - 1, 1 do
				local pPlot = pCity:GetCityIndexPlot(i)
				if pPlot:GetResourceType() == resHorseID and pPlot:GetOwner() == iPlayer then
					numhorsetiles = numhorsetiles +1
					totalhorses = totalhorses + pPlot:GetNumResource()
				end
			end
			local GGbonus = math.floor(math.max(numhorsetiles,totalhorses*GGPointFromXHorse))
			pPlayer:ChangeCombatExperience(GGbonus)
			pCity:SetNumRealBuilding(DummyHorseXP, math.min(XPFromHorseMax,totalhorses*XPFromHorse))
			if pPlayer:IsHuman() and GGbonus>0 then
				local textstrHorse = "+"..GGbonus .. " [ICON_GREAT_GENERAL] from [ICON_RES_HORSES] Horses"
				Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(cityplot:GetX(), cityplot:GetY()))), textstrHorse, 4)
			end
		end
	end
end
				
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(XPandGeneralPointsFromHorses) end;

function ArcherRangeGGAdjacent(iPlayer)
	local pPlayer = Players[iPlayer]
	if (not HasTrait(pPlayer, traitAmazonsID)) then return end
	if pPlayer:IsAlive() then
		for pUnit in pPlayer:Units() do
			local bcheck = false
			local unitplot = pUnit:GetPlot()
			local othercondition = pUnit:GetUnitCombatType()== GameInfoTypes["UNITCOMBAT_MOUNTED"] and pUnit:GetBaseRangedCombatStrength()>0
			--if not pPlayer:IsHuman() then
				if pUnit:GetUnitCombatType()== GameInfoTypes["UNITCOMBAT_ARCHER"] or othercondition then
					for i = 0, unitplot:GetNumUnits() - 1 do
						local wUnit = unitplot:GetUnit(i)
						local iOwner = wUnit:GetOwner()
						if iOwner == iPlayer and wUnit:GetUnitClassType() == GGclassID then
							bcheck = true
							break
						end
					end
					if not bcheck then
						for direction = 0, DirectionTypes.NUM_DIRECTION_TYPES - 1, 1 do
							local pAdjacentPlot = Map.PlotDirection(unitplot:GetX(), unitplot:GetY(), direction)
							if pAdjacentPlot and pAdjacentPlot:IsUnit() then
								for i = 0, pAdjacentPlot:GetNumUnits() - 1 do
									local zUnit = pAdjacentPlot:GetUnit(i)
									local iOwner = zUnit:GetOwner()
									if iOwner == iPlayer and zUnit:GetUnitClassType() == GGclassID then
										bcheck = true
										break
									end
								end
							end	
						end
					end
					pUnit:SetHasPromotion(promoUAGG, bcheck)	
				end
			--end		
			if pUnit:GetUnitType() == iHippomache then
				if pPlayer:GetCurrentEra() == GameInfo.Eras["ERA_CLASSICAL"].ID then
					local strength0 = GameInfo.Units["UNIT_HORSEMAN"].Combat
					pUnit:SetBaseCombatStrength(strength0)
				elseif pPlayer:GetCurrentEra() > GameInfo.Eras["ERA_CLASSICAL"].ID then
					pUnit:SetBaseCombatStrength(17)
				end
			end
		end
	end
end
	
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(ArcherRangeGGAdjacent) end;

function Jar_ArcherRangeGGAdjacentHuman(iPlayer, iUnitID)
	local pPlayer = Players[iPlayer]
	local pUnit = pPlayer:GetUnitByID(iUnitID)
	local unitplot = pUnit:GetPlot()
	if (not HasTrait(pPlayer, traitAmazonsID)) then return end
	if (not pPlayer:IsHuman()) then return end
	if pPlayer:IsAlive() and unitplot then
		local othercondition = pUnit:GetUnitCombatType()== GameInfoTypes["UNITCOMBAT_MOUNTED"] and pUnit:GetBaseRangedCombatStrength()>0
		if pUnit:GetUnitCombatType()== GameInfoTypes["UNITCOMBAT_ARCHER"] or othercondition then -- ranged unit moves
			local bcheck = false
			for i = 0, unitplot:GetNumUnits() - 1 do
				local wUnit = unitplot:GetUnit(i)
				local iOwner = wUnit:GetOwner()
				if iOwner == iPlayer and wUnit:GetUnitClassType() == GGclassID then
					bcheck = true
					break
				end
			end
			if not bcheck then
				for direction = 0, DirectionTypes.NUM_DIRECTION_TYPES - 1, 1 do
					local pAdjacentPlot = Map.PlotDirection(unitplot:GetX(), unitplot:GetY(), direction)
					if pAdjacentPlot and pAdjacentPlot:IsUnit() then
						for i = 0, pAdjacentPlot:GetNumUnits() - 1 do
							local zUnit = pAdjacentPlot:GetUnit(i)
							local iOwner = zUnit:GetOwner()
							if iOwner == iPlayer and zUnit:GetUnitClassType() == GGclassID then
								bcheck = true
								break
							end
						end
					end	
				end
			end
			pUnit:SetHasPromotion(promoUAGG, bcheck)
		elseif pUnit:GetUnitClassType() == GGclassID then -- GG moves
			for potherUnit in pPlayer:Units() do
				local bcheck2 = false
				local otherunitplot = potherUnit:GetPlot()
				local othercondition4 = potherUnit:GetUnitCombatType()== GameInfoTypes["UNITCOMBAT_MOUNTED"] and potherUnit:GetBaseRangedCombatStrength()>0
				if potherUnit:GetUnitCombatType()== GameInfoTypes["UNITCOMBAT_ARCHER"] or othercondition4 then
					if otherunitplot == unitplot then
						bcheck2 = true
					end
					if not bcheck2 then
						for direction = 0, DirectionTypes.NUM_DIRECTION_TYPES - 1, 1 do
							local pAdjacentPlot2 = Map.PlotDirection(otherunitplot:GetX(), otherunitplot:GetY(), direction)
							if pAdjacentPlot2 and pAdjacentPlot2 == unitplot then
								bcheck2 = true
							end
						end
					end	
				end
				potherUnit:SetHasPromotion(promoUAGG, bcheck2)
			end
		else
		end
	end
end

if g_IsTraitActive then GameEvents.UnitSetXY.Add(Jar_ArcherRangeGGAdjacentHuman) end

function AmazonsHorsesAtStart(iPlayer, iCityX, iCityY)
	local pPlayer = Players[iPlayer]
	local iTeam = pPlayer:GetTeam()
	local pPlot = Map.GetPlot(iCityX, iCityY)
	local pCity = pPlot:GetPlotCity()
	local iCity = pCity:GetID()
	if HasTrait(pPlayer, traitAmazonsID) and pCity:IsCapital() then
		local bcheck = true
		for i = 0, pCity:GetNumCityPlots() - 1, 1 do
			local pPlot = pCity:GetCityIndexPlot(i)
			local distance = Map.PlotDistance(iCityX, iCityY, pPlot:GetX(), pPlot:GetY())
			if pPlot and pPlot:GetResourceType() == resHorseID and distance<=2 then
				bcheck = false
				break
			end
		end
		if bcheck then
			for i = 0, pCity:GetNumCityPlots() - 1, 1 do
				local pPlot = pCity:GetCityIndexPlot(i)
				local distance = Map.PlotDistance(iCityX, iCityY, pPlot:GetX(), pPlot:GetY())
				local ownercondition = (pPlot:GetOwner() == iPlayer or pPlot:GetOwner() == -1)
				if pPlot and ownercondition and distance<=2 then
					if pPlot:GetResourceType() == -1 and pPlot:GetFeatureType() == -1 and not pPlot:IsCity() and not pPlot:IsWater() and not pPlot:IsMountain() then
						pPlot:SetResourceType(resHorseID, 4)
						print("Horses at start.")
						break
					end
				end
			end		
		end
	end
end

if g_IsTraitActive then GameEvents.PlayerCityFounded.Add(AmazonsHorsesAtStart) end		
--==========================================================================================================================
-- UB: HEMP SMOKE TENT
--========================================================================================================================
function AmazonUBEffects(iPlayer)
	local pPlayer = Players[iPlayer]
	local numturns = math.floor((CannabisPromoTurns)*iMod)
	if (HasTrait(pPlayer, traitAmazonsID) and pPlayer:IsAlive()) then
		for pUnit in pPlayer:Units() do
			local unitplot = pUnit:GetPlot()
			NoCannabisPromoTurns = load( pUnit, "CannabisPromoTurns" );
			if unitplot:IsCity() and pUnit:GetDomainType() == DomainTypes.DOMAIN_LAND then
				local pCity = unitplot:GetPlotCity()
				if pCity:IsHasBuilding(iHempSmokeTent) then
					if not isUsingCBO then
						pUnit:ChangeDamage(-5)
					end
					save( pUnit, "CannabisPromoTurns", numturns );
					pUnit:SetHasPromotion(promoCannabis, true)
				end
			end
			if NoCannabisPromoTurns ~= nil then
				if NoCannabisPromoTurns >0 then
					save( pUnit, "CannabisPromoTurns", NoCannabisPromoTurns -1);
					pUnit:SetHasPromotion(promoCannabis, true)
				else
					pUnit:SetHasPromotion(promoCannabis, false)
				end
			end
		end
	end
end
	
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(AmazonUBEffects) end;


if isUsingCBO then
local eResourceKannabis = GameInfoTypes.RESOURCE_JAR_KANNABIS
local eImprovementPlantation = GameInfoTypes.IMPROVEMENT_PLANTATION

-- creates Kannabis under city
function OnConstructionPlaceKannabis(iPlayer, iCity, iBuilding)
	local pPlayer = Players[iPlayer]
	
	if not(pPlayer and HasTrait(pPlayer, traitAmazonsID)) then return end
	if iBuilding ~= iHempSmokeTent then return end
	
	local pCity = pPlayer:GetCityByID(iCity)
	local pPlot = Map.GetPlot(pCity:GetX(), pCity:GetY())
	Jar_PlaceResourceOnPlot(iPlayer, eResourceKannabis, 1, pPlot, eImprovementPlantation)
end

-- deletes Kannabis if city is razed
function OnRazeRemoveKannabis(hexPos, iOldOwner, iCity, iNewOwner)
	local pPlayer = Players[iOldOwner]
	local pCity = pPlayer:GetCityByID(iCity)
	if pCity then
		local pPlot = Map.GetPlot(pCity:GetX(), pCity:GetY())
		if pPlot and pPlot:GetResourceType() == eResourceKannabis then
			pPlot:SetResourceType(-1)
		end
	end
end

if g_IsTraitActive then
	GameEvents.CityConstructed.Add(OnConstructionPlaceKannabis)
	Events.SerialEventCityDestroyed.Add(OnRazeRemoveKannabis)
end

end
--==========================================================================================================================
-- UU: HIPPOMACHE
--========================================================================================================================
function Jar_CanTrapHorses(unit)
	local pBool = false;
	local pPlot = unit:GetPlot();
	if unit:MovesLeft() <= 1 then return pBool end
	if pPlot ~= nil then
		local iOwner = pPlot:GetOwner()
		if iOwner ~= -1 then return pBool end
		if pPlot:GetResourceType() == resHorseID then
			pBool = true
		end
	end
	return pBool
end

function Jar_DoTrapHorses(pPlayer, pUnit) 
	local pUnitPlot = pUnit:GetPlot();
	local pTeam = Teams[pPlayer:GetTeam()]
	local numres = pUnitPlot:GetNumResource()
	local bcheck = true
	pUnit:SetHasPromotion(promoHorseTrapper, false)
	for cCity in pPlayer:Cities() do
        local pCityPlot = cCity:Plot()
        local distance = Map.PlotDistance(pCityPlot:GetX(), pCityPlot:GetY(), pUnitPlot:GetX(), pUnitPlot:GetY())
        if distance <= 3 then
			pUnitPlot:SetOwner(pPlayer:GetID(), cCity:GetID(), true, true);
			pUnitPlot:SetRevealed(pTeam, true)
			pUnitPlot:SetImprovementType( pastureID )
			bcheck = false
			break
        end
    end
	if bcheck then
		save( pUnit, "NumHorsesTrapped", numres );
		pUnitPlot:SetResourceType(-1)
		pUnit:SetHasPromotion(promoHorseTrappeD, true)
	end
end

function Jar_CanPlaceHorses(unit) -- checks if a unit is on a valid plot
	local pBool = false;
	local pPlot = unit:GetPlot();
	if unit:MovesLeft() <= 1 then return pBool end
	if pPlot ~= nil then
		local iOwner = pPlot:GetOwner()
		if unit:GetOwner() ~= iOwner then return pBool end
		if pPlot:GetResourceType() == -1 and not pPlot:IsWater() and not pPlot:IsMountain() then
			pBool = true
		end
	end
	return pBool
end

function Jar_DoPlaceHorses(pPlayer, pUnit)
	local pUnitPlot = pUnit:GetPlot();
	local pTeam = Teams[pPlayer:GetTeam()]	
	pUnit:SetHasPromotion(promoHorseTrappeD, false)
	if pUnitPlot:GetFeatureType() ~= -1 then pUnitPlot:SetFeatureType(-1) end
	NumHorsesTrapped = load( pUnit, "NumHorsesTrapped" );
	pUnitPlot:SetResourceType(resHorseID, NumHorsesTrapped)
	pUnitPlot:SetImprovementType(pastureID)
	save( pUnit, "NumHorsesTrapped", nil );
end

function Jar_AITrapPlaceHorses(playerID)
	local aPlayer = Players[playerID]
	local aTeam = Teams[aPlayer:GetTeam()]
	if aPlayer:IsHuman() then return end
	if aPlayer:IsAlive() then
		for aUnit in aPlayer:Units() do
			if aUnit:IsHasPromotion(promoHorseTrapper) then
				local aUnitPlot = aUnit:GetPlot();
				local iOwner = aUnitPlot:GetOwner()
				if iOwner ~= -1 then return end
				if aUnitPlot:GetResourceType() ~= resHorseID then return end
				local numres = aUnitPlot:GetNumResource()
				local aCity = Neirai_GetNearestCity(aPlayer, aUnitPlot)
				local aCityPlot = aCity:Plot()
				local distance = Map.PlotDistance(aCityPlot:GetX(), aCityPlot:GetY(), aUnitPlot:GetX(), aUnitPlot:GetY())
				if distance <=3 then
					aUnitPlot:SetOwner(playerID, aCity:GetID(), true, true);
					aUnitPlot:SetRevealed(aTeam, true)
					aUnitPlot:SetImprovementType( pastureID )
				else
					for i = 0, aCity:GetNumCityPlots() - 1, 1 do
						local pPlot = aCity:GetCityIndexPlot(i)
						if pPlot and pPlot:GetOwner() == playerID and pPlot:GetResourceType() == -1 and not pPlot:IsWater() and not pPlot:IsMountain() and not pPlot:IsCity() then
							if pPlot:GetFeatureType() ~= -1 then pPlot:SetFeatureType(-1) end
							pPlot:SetResourceType(resHorseID, math.max(4, numres))
							pPlot:SetImprovementType(pastureID)
							break
						end
					end		
				end
				aUnit:SetHasPromotion(promoHorseTrapper, false)
			end
		end
	end
end

if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(Jar_AITrapPlaceHorses) end
--==========================================================================================================================
-- UI FUNCTIONS
--==========================================================================================================================
-------------------------------------------------------------------------------------------------------------------------
-- UNIT PANEL
-------------------------------------------------------------------------------------------------------------------------
local primaryStack = ContextPtr:LookUpControl("/InGame/WorldView/UnitPanel/PrimaryStack") 
local isEUIactive  = (primaryStack == nil)

if (not isUsingCBP) then
if (not isEUIactive) then
--Jar_Hippomache_UI_UnitActionButton
function Jar_TrapHorses_UI_UnitActionButton()
	local unit = UI.GetHeadSelectedUnit();
	Jar_DoTrapHorses(activePlayer, unit)
end

function Jar_PlaceHorses_UI_UnitActionButton()
	local unit = UI.GetHeadSelectedUnit();
	Jar_DoPlaceHorses(activePlayer, unit)
end

--Jar_Hippomache_UI_SerialEventUnitInfoDirty
function Jar_TrapHorses_SerialEventUnitInfoDirty()
	--if (not HasTrait(activePlayer, traitAmazonsID)) then return end

	local unit = UI.GetHeadSelectedUnit();
	if (not unit) then return end
	if unit:IsHasPromotion(promoHorseTrapper) then
		local bcheck = not Jar_CanTrapHorses(unit)
		Controls.UnitActionButton:SetHide(bcheck) -- if not allowed -> hide true; if allowed -> hide false
	end
	
	local buildCityButtonActive = unit:IsFound();				
	
	local primaryStretchy = ContextPtr:LookUpControl("/InGame/WorldView/UnitPanel/PrimaryStretchy")	
	primaryStack:CalculateSize();
	primaryStack:ReprocessAnchoring();
    
	local stackSize = primaryStack:GetSize();
	local stretchySize = primaryStretchy:GetSize();
	local buildCityButtonSize = 0
	if buildCityButtonActive then
		if OptionsManager.GetSmallUIAssets() and not UI.IsTouchScreenEnabled() then
			buildCityButtonSize = 36;
		else
			buildCityButtonSize = 60;
		end
	end
	primaryStretchy:SetSizeVal( stretchySize.x, stackSize.y + buildCityButtonSize + 348 );
end

function Jar_PlaceHorses_UI_SerialEventUnitInfoDirty()
	--if (not HasTrait(activePlayer, traitAmazonsID)) then return end

	local unit = UI.GetHeadSelectedUnit();
	if (not unit) then return end
	if unit:IsHasPromotion(promoHorseTrappeD) then
		local bcheck = not Jar_CanPlaceHorses(unit)
		Controls.UnitActionButton2:SetHide(bcheck) -- if not allowed -> hide true; if allowed -> hide false
	end
	
	local buildCityButtonActive = unit:IsFound();				
	
	local primaryStretchy = ContextPtr:LookUpControl("/InGame/WorldView/UnitPanel/PrimaryStretchy")	
	primaryStack:CalculateSize();
	primaryStack:ReprocessAnchoring();
    
	local stackSize = primaryStack:GetSize();
	local stretchySize = primaryStretchy:GetSize();
	local buildCityButtonSize = 0
	if buildCityButtonActive then
		if OptionsManager.GetSmallUIAssets() and not UI.IsTouchScreenEnabled() then
			buildCityButtonSize = 36;
		else
			buildCityButtonSize = 60;
		end
	end
	primaryStretchy:SetSizeVal( stretchySize.x, stackSize.y + buildCityButtonSize + 348 );
end

--Jar_Hippomache_UI_UpdateUnitInfoPanel
local function Jar_TrapHorses_UpdateUnitInfoPanel()
	if (not OptionsManager.GetSmallUIAssets()) then
		Controls.UnitActionButton:SetSizeVal(50,50)
		Controls.UnitActionIcon:SetSizeVal(64,64)
		Controls.UnitActionIcon:SetTexture("Jar_Hippomache_Mission64.dds")
	else
		Controls.UnitActionButton:SetSizeVal(36,36)
		Controls.UnitActionIcon:SetSizeVal(45,45)
		Controls.UnitActionIcon:SetTexture("Jar_Hippomache_Mission45.dds")
	end
	Controls.UnitActionIcon:LocalizeAndSetToolTip("TXT_KEY_JAR_HIPPOMACHE_TRAP_HORSES")
	Controls.UnitActionButton:ChangeParent(ContextPtr:LookUpControl("/InGame/WorldView/UnitPanel/PrimaryStack"))
end

local function Jar_PlaceHorses_UI_UpdateUnitInfoPanel()
	if (not OptionsManager.GetSmallUIAssets()) then
		Controls.UnitActionButton2:SetSizeVal(50,50)
		Controls.UnitActionIcon2:SetSizeVal(64,64)
		Controls.UnitActionIcon2:SetTexture("Jar_Hippomache_Mission64.dds")
	else
		Controls.UnitActionButton2:SetSizeVal(36,36)
		Controls.UnitActionIcon2:SetSizeVal(45,45)
		Controls.UnitActionIcon2:SetTexture("Jar_Hippomache_Mission45.dds")
	end
	Controls.UnitActionIcon2:LocalizeAndSetToolTip("TXT_KEY_JAR_HIPPOMACHE_PLACE_HORSES")
	Controls.UnitActionButton2:ChangeParent(ContextPtr:LookUpControl("/InGame/WorldView/UnitPanel/PrimaryStack"))
end

local function Initialize()	
	Events.LoadScreenClose.Add(Jar_TrapHorses_UpdateUnitInfoPanel);
	Events.SerialEventUnitInfoDirty.Add(Jar_TrapHorses_SerialEventUnitInfoDirty);
end
local function Initialize2()	
	Events.LoadScreenClose.Add(Jar_PlaceHorses_UI_UpdateUnitInfoPanel);
	Events.SerialEventUnitInfoDirty.Add(Jar_PlaceHorses_UI_SerialEventUnitInfoDirty);
end
IconHookup(29, 45, "UNIT_ACTION_ATLAS", Controls.UnitActionIcon)	
Controls.UnitActionButton:RegisterCallback(Mouse.eLClick, Jar_Hippomache_UI_UnitActionButton);
Initialize();

IconHookup(29, 45, "UNIT_ACTION_ATLAS", Controls.UnitActionIcon)	
Controls.UnitActionButton2:RegisterCallback(Mouse.eLClick, Jar_Hippomache_UI_UnitActionButton);
Initialize2();

else
--==========================================================================================================================
-- ENHANCED USER INTERFACE COMPATIBILITY
--==========================================================================================================================
IconHookup(29, 45, "UNIT_ACTION_ATLAS", Controls.TomButtonImage)
Controls.TomButtonBackground:SetHide(true)
local pSelUnit
function Jar_HippomacheSelection1(playerID, unitID, x, y, a5, bool)
	if bool then
		local pPlayer = Players[playerID]
		local pUnit = pPlayer:GetUnitByID(unitID);
		if pUnit:IsHasPromotion(promoHorseTrapper) then
			ContextPtr:SetHide(false)
			Controls.TomButtonBackground:SetHide(false)
			pSelUnit = pUnit;
			Controls.TomButton:SetDisabled(false)
			Controls.TomButton:RegisterCallback(Mouse.eLClick, Jar_DoPerformAction1)	
			Controls.TomButton:LocalizeAndSetToolTip("TXT_KEY_JAR_HIPPOMACHE_TRAP_HORSES")
		end
	else
		Controls.TomButtonBackground:SetHide(true)
		pSelUnit = nil;
	end
end
Events.UnitSelectionChanged.Add(Jar_HippomacheSelection1)


IconHookup(29, 45, "UNIT_ACTION_ATLAS", Controls.TomButtonImage2)
Controls.TomButtonBackground2:SetHide(true)
local pSelUnit2
function Jar_HippomacheSelection2(playerID, unitID, x, y, a5, bool)
	if bool then
		local pPlayer = Players[playerID]
		local pUnit = pPlayer:GetUnitByID(unitID);
		if pUnit:IsHasPromotion(promoHorseTrappeD) then
			ContextPtr:SetHide(false)
			Controls.TomButtonBackground2:SetHide(false)
			pSelUnit2 = pUnit;
			Controls.TomButton2:SetDisabled(false)
			Controls.TomButton2:RegisterCallback(Mouse.eLClick, Jar_DoPerformAction2)	
			Controls.TomButton2:LocalizeAndSetToolTip("TXT_KEY_JAR_HIPPOMACHE_PLACE_HORSES")
		end
	else
		Controls.TomButtonBackground2:SetHide(true)
		pSelUnit2 = nil;
	end
end
Events.UnitSelectionChanged.Add(Jar_HippomacheSelection2)

function Jar_DoPerformAction1(playerID, unitID)
	local hPlayer = Players[playerID]
	local hTeam = hPlayer:GetTeam()
	local hUnit = pSelUnit;
	local hUnitPlot = hUnit:GetPlot();
	if Jar_CanTrapHorses(hUnit) then
		Jar_DoTrapHorses(hPlayer, hUnit)
		Controls.TomButtonBackground:SetHide(true)
		Controls.TomButton:SetDisabled(true)
	else
		local iX = hUnit:GetX()
		local iY = hUnit:GetY()
		Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(iX, iY))), "[COLOR_WARNING_TEXT]" .. Locale.ConvertTextKey("TXT_KEY_MISSION_JAR_HIPPOMACHE_TRAP_HORSES_DISABLED") .. "[ENDCOLOR]", 0)
	end
end

function Jar_DoPerformAction2(playerID, unitID)
	local hPlayer = Players[playerID]
	local hTeam = hPlayer:GetTeam()
	local hUnit = pSelUnit2;
	local hUnitPlot = hUnit:GetPlot();
	if Jar_CanPlaceHorses(hUnit) then
		Jar_DoPlaceHorses(hPlayer, hUnit)
		Controls.TomButtonBackground2:SetHide(true)
		Controls.TomButton2:SetDisabled(true)
	else
		local iX = hUnit:GetX()
		local iY = hUnit:GetY()
		Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(iX, iY))), "[COLOR_WARNING_TEXT]" .. Locale.ConvertTextKey("TXT_KEY_MISSION_JAR_HIPPOMACHE_PLACE_HORSES_DISABLED") .. "[ENDCOLOR]", 0)
	end
end

end
else
--==========================================================================================================================
-- CBP MISSION EVENTS COMPATIBILITY
--==========================================================================================================================
local function Jar_TrapHorses_CustomMissionStart(playerID, unitID, missionID, data1, data2, flags, turn)
	local player = Players[playerID]
	local unit = player:GetUnitByID(unitID)
	local plot = unit:GetPlot()
	if missionID == missionHippomacheTrapHorses then
		Jar_DoTrapHorses(player, unit)
		return CUSTOM_MISSION_ACTION
	end
	return CUSTOM_MISSION_NO_ACTION
end

local function Jar_PlaceHorses_CustomMissionStart(playerID, unitID, missionID, data1, data2, flags, turn)
	local player = Players[playerID]
	local unit = player:GetUnitByID(unitID)
	local plot = unit:GetPlot()
	if missionID == missionHippomachePlaceHorses then
		Jar_DoPlaceHorses(player, unit)
		return CUSTOM_MISSION_ACTION
	end
	return CUSTOM_MISSION_NO_ACTION
end

-- Jar_Hippomache_CustomMissionCompleted
local function Jar_TrapHorses_CustomMissionCompleted(playerID, unitID, missionID, data1, data2, flags, turn)
	return (missionID == missionHippomacheTrapHorses)
end

local function Jar_PlaceHorses_CustomMissionCompleted(playerID, unitID, missionID, data1, data2, flags, turn)
	return (missionID == missionHippomachePlaceHorses)
end

-- Jar_Hippomache_CustomMissionPossible
local function Jar_TrapHorses_CustomMissionPossible(playerID, unitID, missionID, data1, data2, _, _, plotX, plotY, bTestVisible)
	if ((not playerID) or (not unitID) or (not missionID)) then return end
	local player = Players[playerID]
	local unit = player:GetUnitByID(unitID)
	if missionID == missionHippomacheTrapHorses then
		if unit:IsHasPromotion(promoHorseTrapper) then
			return Jar_CanTrapHorses(unit)
		end
	end	
	return false
end

local function Jar_PlaceHorses_CustomMissionPossible(playerID, unitID, missionID, data1, data2, _, _, plotX, plotY, bTestVisible)
	if ((not playerID) or (not unitID) or (not missionID)) then return end
	local player = Players[playerID]
	local unit = player:GetUnitByID(unitID)
	if missionID == missionHippomachePlaceHorses then
		if unit:IsHasPromotion(promoHorseTrappeD) then
			return Jar_CanPlaceHorses(unit)
		end
	end	
	return false
end

GameEvents.CustomMissionStart.Add(Jar_TrapHorses_CustomMissionStart)
GameEvents.CustomMissionCompleted.Add(Jar_TrapHorses_CustomMissionCompleted)
GameEvents.CustomMissionPossible.Add(Jar_TrapHorses_CustomMissionPossible)
GameEvents.CustomMissionStart.Add(Jar_PlaceHorses_CustomMissionStart)
GameEvents.CustomMissionCompleted.Add(Jar_PlaceHorses_CustomMissionCompleted)
GameEvents.CustomMissionPossible.Add(Jar_PlaceHorses_CustomMissionPossible)
end
--=======================================================================================================================
--TROLLER'S ADDITIONAL ACHIEVEMENTS
--=======================================================================================================================
local TAAid = "432bc547-eb05-4189-9e46-232dbde8f09f"
local isAAActive = Game_IsUsingMod(TAAid)
local iCargoShip = GameInfoTypes["UNIT_CARGO_SHIP"]
----------------------------------------
function UnlockAmazonsAchievement(iPlayer)
	local pPlayer = Players[iPlayer]
	if IsAAUnlocked('AA_JAR_AMAZONS_SPECIAL') then return end
	if (HasTrait(pPlayer, traitAmazonsID) and pPlayer:IsAlive()) then
		if pPlayer:GetNumResourceAvailable(resHorseID, false) >= 30 then
			UnlockAA('AA_JAR_AMAZONS_SPECIAL')
		end
	end
end
if isAAActive then
	GameEvents.PlayerDoTurn.Add(UnlockAmazonsAchievement)
end
---------------------------------------------------------
print("Amazons functions loaded")
--==========================================================================================================================
--==========================================================================================================================